/***************************************************************************
 *            convwindow.h
 *
 *  Fri Jan 21 23:21:00 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef CONVWINDOW_H
#define CONVWINDOW_H
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "basicwindow.h"
#include "processingani.h"
#include "settings.h"
#include "utilities.h"
#include "userutilities.h"
#include "contactsselection.h"
#include "gettext.h"
#include "systemcharset.h"
#include "usertooltips.h"



// message types that can be displayed
enum MessageType
{
	MT_INCOMING,
	MT_OUTGOING,
	MT_NOTIFICATION,
	MT_HINCOMING,
	MT_HOUTGOING,
	MT_ERROR	
};



// toolbar button identifiers
enum ConversationToolTypes
{
	TOOL_NONE,
	// info tools
	TOOL_STATUS,
	TOOL_TIME,
	// tool buttons
	TOOL_ENCODING,
	TOOL_ENCRYPT,
	TOOL_HISTORY,
	TOOL_INFO,
	TOOL_SMILEYS,
	TOOL_SENDFILE,
	TOOL_SENDLINK,
	TOOL_STARTCHAT,
	TOOL_FGCOLOR,
	TOOL_BGCOLOR,
	TOOL_SENDSERVER,
	TOOL_SENDURGENT,
	TOOL_SEND,
	TOOL_MULTIPLE,

	// graphical elements
	TOOL_SENDANI,
	TOOL_SEPARATOR,
	TOOL_ALIGNLEFT,
	TOOL_ALIGNRIGHT,
	
	// new tools
	TOOL_TOGGLESMILEYS,
	TOOL_TOGGLEURL,
	
	NUM_TOOLS
};



// a toolbar item
typedef struct ToolbarItem
{
	char* 		tip;
	gchar* 		stockIcon;
	gint 		id;
	gboolean    alignLeft;
};


typedef struct SmallHistoryEntry
{
	gchar			*msg;
	gchar			*url;
	time_t			time;
	MessageType 	type;
	IMUserDaemon 	*user;
};



class ConversationWindow: public BasicWindow
{
	public:
		ConversationWindow();
		virtual ~ConversationWindow();
	
		virtual void windowDeactivated();
		virtual void windowActivated();
	
		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);
		virtual void	 SettingsChangedCallback(gchar* sname, GList *values);
	
		void	enableURLMode();
		void	disableURLMode();
	
		void	updateTitle();
	
		void	setTypedText(gchar* text);
	
		GtkWidget *createToolbar(gchar *description, guint spacing, gboolean preview=FALSE, GList **entries = NULL);
	
	protected:
	
		GList		*smallHistory;
	
		GtkWidget	*mainHBox,
					*localRemoteDivider,
					*localView,
					*remoteView,
					*sendButton,
					*multipleButton,
					*encryptButton,
					*toggleSmileysImg,
					*toggleURLImg,
					*serverCheckbox,
					*urgentCheckbox,
					*tabLabelImage,
					*tabLabelText,
					*statusEntry,
					*timeEntry,
					*typingImage,
					*smileysMenu,
					*URLbox,
					*URLentry,
					*charsetMenu,
					*goToButton;
	
		GtkWidget	*toolbar1Container,
					*toolbar2Container,
					*toolbar3Container;
	
	
		guint		timeTimeoutID;
		guint		oldStatus;
		time_t		keyPressTime;
		gboolean	isURLMode;
		gchar		*hoveredLink;
		gchar		*toSendText,
					*toSendLink,
					*preferredEncoding;
					
		gchar		*messageTag;
					
		gboolean	interpretSmileys,
					interpretURIs,
					dontApplyColors,
					sendTyping,
					useRealName,
					sendByReturn,
					autoSendServer;
		gulong		autoSendServerTime;
		guint		autoSendServerTimeoutID;
					
		GdkColor	tabMessageColor,
					tabTypingColor;
					
	
		ProcessingAni	*sendAni;
		PangoAttribute	*tabLabelColor;
		
		ContactsSelection* multipleSelection;
		UserTooltips	 * tooltips;
	
		virtual GtkWidget* createWindowContent();
		virtual void	   destroyWindowContent();
		virtual GtkWidget* createContainerTabWidget();
	
		void	sendMessage();
		void	sendMessageMultiple(gint count);
		void	finishSendMessage(MessageEventInfo* messageEvent, gint result);
		void	stopSendMessage();
		
		void	insertMessage(time_t msgTime, gchar* msg, gchar* url, IMUserDaemon* user, MessageType msgType, gboolean add2History);
	
		void	selectColor(gboolean foreground);
		void	updateLocalColors();
		void	setTabWidgetColor(GdkColor* col);
	
		// helper functions for creating a window
		void		createCharsetList();
		
		GtkWidget	*createRemoteEntryBox();
		GtkWidget	*createLocalEntryBox();
		void		createToolbars();
		
		void		createSmileysMenu();
		
		void		showOldEntries(gint toSkip);
		
		void		rebuildMessageView();
		
		void		escPressed();
						
		// multiple selection functions
		void		showMultipleSelectionList();
		void		hideMultipleSelectionList();
		
		// callbacks		
		static void cb_sendButtonClicked(ConversationWindow *cw);
		static void cb_sendServerButtonClicked(ConversationWindow *cw);
		static void cb_toolbarClicked(GtkWidget* clickedWidget, ConversationWindow *cw);
		static void cb_multipleSendButtonClicked(ConversationWindow *self);
		
		static gboolean cb_keyPressed(ConversationWindow *cw, GdkEventKey *event);
		static void		cb_localEntryChanged(ConversationWindow *self);
		
		static gboolean cb_setUserTime(ConversationWindow *cw);
		
		static void		cb_colorSelectionCheckboxClicked(GtkWidget* w, GtkWidget* d);
		
		static gboolean cb_smileysMenuKeyReleased(GtkWidget *w, GdkEventKey *event, GtkWidget **toDestroy);
		static void		cb_smileyClicked(GtkWidget* w, ConversationWindow *cw);
		
		static gboolean	cb_remoteViewMouseMoved(GtkWidget* v, GdkEventMotion* event, ConversationWindow *self);
		static gboolean cb_remoteViewVisibilityChanged(GtkWidget* v, GdkEventVisibility* event, ConversationWindow *self);

		static gboolean cb_remoteViewURLPressed(ConversationWindow* self, GdkEventButton* event);
		static void		cb_popupPopulated(ConversationWindow* self, GtkMenu* menu);

		static void		cb_copyToCBItemClicked(ConversationWindow* self);
		static void		cb_openURLItemClicked(ConversationWindow* self);

		static void		cb_charsetListItemClicked(GtkWidget *w, ConversationWindow *self);
		
		static gboolean	cb_urlEntryTyped(ConversationWindow* self);
		static void		cb_openURLButtonClicked(ConversationWindow* self);
		
		static IMUserDaemon* cb_tooltipsFunction(GtkWidget* w, GdkEventMotion *event, ConversationWindow* self);

		static gboolean cb_autoSendServerTimeout(ConversationWindow* self);
};

#endif
