/***************************************************************************
 *            optionswindow.h
 *
 *  Fri Jul 29 19:58:21 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef _OPTIONSWINDOW_H
#define _OPTIONSWINDOW_H

#include "imuserdaemon.h"
#include "imownerdaemon.h"
#include "basicwindow.h"
#include "settings.h"
#include "gettext.h"
#include "convwindow.h"
#include "utilities.h"
#include "mainwindow.h"

#include "eggcellrendererkeys.h"

class OptionsWindow: public BasicWindow
{
	public:
		OptionsWindow();
		~OptionsWindow();
		
	private:
		GtkListStore	*itemsStore;	
	
		GtkWidget		*notebook,
						*itemsView,
						*applyButton;
	
		GList			*optionsItems;
	
		GtkWidget	*createWindowContent();
		void		destroyWindowContent();
	
		GtkWidget	*createButtonbar();
	
		// callbacks
		static void cb_optionsItemSelected(OptionsWindow* self);
		static void cb_applyButtonClicked(OptionsWindow* self);
};


class OptionsWindowItem
{
	public:
		GdkPixbuf	*titleImage;
		gchar		*titleString;	
	
		GtkWidget	*tabWidget,
					*parentContainer,
					*window;
	
		gboolean	hasOwnNotebook;

	
		OptionsWindowItem();
		virtual ~OptionsWindowItem();
	
		virtual GtkWidget	*createTab();
		virtual void		destroyTab();
		virtual gboolean	applyChanges();
};


class OptionsWindowItem_IMContactList: public OptionsWindowItem
{
	public:
		OptionsWindowItem_IMContactList();
		~OptionsWindowItem_IMContactList();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:

	GtkWidget		*sortOrderList,
					*blinkByOnlineCheckbox,
					*blinkByARCheckbox,
					*blinkEventsCheckbox,
					*detailledModeCheckbox,
					*useColorsCheckbox,
					*useRealNamesCheckbox,
					*showExtendedCheckbox,
					*showTipsCheckbox,
					*tipStatusCheckbox,
					*tipRealnameCheckbox,
					*tipEmailCheckbox,
					*tipClientNameCheckbox,
					*tipIPCheckbox,
					*tipOnlineSinceCheckbox,
					*tipARCheckbox,
					*tipIDLETimeCheckbox,
					*tipProtocolCheckbox,
					*tipIDCheckbox,
					*vlistColorButton,
					*noMsgColorButton,
					*idleColorButton,
					*groupsColorButton,
					*extendedAResponseCheckbox,
					*extendedBirthdayCheckbox,
					*extendedCellCheckbox,
					*extendedPhoneCheckbox,
					*extendedTypingCheckbox,
					*extendedEncryptCheckbox,
					*contactsFontButton,
					*useOwnFontCheckbox;
					
		GtkWidget	*sortUpButton,
					*sortDownButton;
					
		GtkListStore	*sortStore;	

		GtkWidget	*createGeneralTab();	
		GtkWidget	*createColorsLookTab();
		void		moveSelectedCriteria(gboolean moveUp);
		
		
		// callbacks
		static void cb_sortUpButtonClicked(OptionsWindowItem_IMContactList* self);
		static void cb_sortDownButtonClicked(OptionsWindowItem_IMContactList* self);
		static void cb_sortListCursorChanged(OptionsWindowItem_IMContactList* self);
		static void cb_sortListEnabledToggled(GtkCellRendererToggle* t, gchar* pathStr, OptionsWindowItem_IMContactList* self);
};



class OptionsWindowItem_MainWindow: public OptionsWindowItem
{
	public:
		OptionsWindowItem_MainWindow();
		~OptionsWindowItem_MainWindow();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget	*showMainMenuCheckbox,
					*showNotifyBoxCheckbox,
					*showStatusButtonsCheckbox,
					*createWindowBorderCheckbox,
					*alwaysOnTopCheckbox,
					*dockWindowCheckbox,
					*dockLeftRadiobox,
					*dockRightRadiobox,
					*inTaskbarMaximizedCheckbox,
					*inTaskbarMinimizedCheckbox,
					*inPagerCheckbox,
					*titleShowStarCheckbox,
					*titleShowOwnersCheckbox,
					*showWindowOnAllCheckbox,
					*dockDoubleClickCheckbox;
};



struct EditToolbarElements
{
	GList			*list;
	GtkListStore	*store;
	gchar			*rules;
	gulong			spacing;
	gboolean		visible;
	GtkWidget		*container;
};


class OptionsWindowItem_msgWindow: public OptionsWindowItem
{
	public:
		OptionsWindowItem_msgWindow();
		~OptionsWindowItem_msgWindow();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		EditToolbarElements	localbar,
						    remotebar,
							buttonbar,
							*currentbar;
	
	
		GtkListStore *toolsStore;
	
		GtkWidget	*editToolsView,
					*toolsView,
					*editToolsSelection,
					*showToolbarCheckbox,
					*toolbarSpacingSpinButton,
					*addElementButton,
					*removeElementButton,
					*remotePreviewView,
					*localPreviewView;
	
	GtkWidget		*sendTypingCheckbox,
					*showSmileysCheckbox,
					*showURIsCheckbox,
					*showHistoryCheckbox,
					*historyCountBox,
					*recColorButton,
					*sentColorButton,
					*msgColorButton,
					*typeColorButton,
					*dontApplyColorsButton,
					*msgFormatEntry,
					*msgFontButton,
					*encodingComboBox,
					*sendByReturnCheckbox,
					*useMsgFontCheckbox;
	
		GdkColor	incomingCol,
					outgoingCol,
					msgCol,
					typeCol;
	
		gchar		*messageTag,
					*standardEncoding,
					*msgFont;
		
		gboolean	dontApplyColors,
					sendTyping,
					showSmileys,
					showURIs,
					showHistory,
					sendByReturn,
					useMsgFont;
					
		gulong		historyCount;
	
		GtkTreeViewColumn	*editToolsColumn;

		void 		createPreviewToolbar(EditToolbarElements* tb);
		GtkWidget 	*createPreview();
		GtkWidget	*createGeneralTab();
		GtkWidget	*createColorsTab();
		GtkWidget	*createToolbarsTab();
		GtkListStore	*createToolbarStore(GList	*el);
		
		void	refreshRemoteViewPreview();
		void	refreshMessageFormatList(gchar* tag, gchar* alias, gchar* firstname, gchar* lastname, GtkTextBuffer* buf, gchar* dirTag);
				
		static void cb_toolbarSelected(OptionsWindowItem_msgWindow* self);
		static void cb_addElementButtonClicked(OptionsWindowItem_msgWindow* self);
		static void cb_removeElementButtonClicked(OptionsWindowItem_msgWindow* self);
		static void cb_showToolbarBoxClicked(OptionsWindowItem_msgWindow* self);
		static void cb_msgRefreshButtonClicked(OptionsWindowItem_msgWindow* self);
		static void cb_msgHelpButtonClicked(OptionsWindowItem_msgWindow* self);
		
		static void	cb_msgColorSelected(GtkColorButton* but, OptionsWindowItem_msgWindow* self);
		static void cb_dontApplyColorsButtonClicked(OptionsWindowItem_msgWindow* self);
};


class OptionsWindowItem_extensions: public OptionsWindowItem
{
	public:
		OptionsWindowItem_extensions();
		~OptionsWindowItem_extensions();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget	*useGnomeCheckbox,
					*openURLinTabRadiobox,
					*openURLinWndRadiobox,
					*openURLinCurRadiobox,
					*browserCombobox,
					*ownBrowserEntry,
					*ownBrowserHBox,
					*ownBrowserInfoLabel;
	
		u_browserInfo	*createBrowsersBox(gchar* browserCommand);
	
		static void cb_browserComboboxChanged(OptionsWindowItem_extensions* self);
};


class OptionsWindowItem_onEvent: public OptionsWindowItem
{
	public:
		OptionsWindowItem_onEvent();
		~OptionsWindowItem_onEvent();	
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget	*paramBox,
					*acceptBox,
					*generalBox;
	
		GtkWidget	*enableCheckbox,
					*commandEntry,
					*paramMsgEntry,
					*paramURLEntry,
					*paramChatEntry,
					*paramFTEntry,
					*paramONEntry,
					*paramSysMsgEntry,
					*paramMsgSentEntry,
					*acceptAwayCheckbox,
					*acceptNACheckbox,
					*acceptOccCheckbox,
					*acceptDnDCheckbox,
					*acceptLogonCheckbox;
};



struct EditServerConTab
{
	IMPluginDaemon	*proto;
	GtkWidget		*server,
					*port;
};


class OptionsWindowItem_connections: public OptionsWindowItem
{
	public:
		OptionsWindowItem_connections();
		~OptionsWindowItem_connections();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GList	*serverTabs;	
	
		GtkWidget	*fwEnableCheckbox,
					*fwDirectConnectionsCheckbox,
					*fwPortsFromSpinButton,
					*fwPortsToSpinButton,
					*pxUseCheckbox,
					*pxUseAuthorizationCheckbox,
					*pxServerEntry,
					*pxPortSpinButton,
					*pxUsernameEntry,
					*pxPasswordEntry,
					*kcIDClashCheckbox,
					*kcDisconnectCheckbox,
					*kcDisconnectSpinButton,
					*shAlwaysServerCheckbox,
					*shAutoServerCheckbox,
					*shAutoServerSpinButton,
					*notebook;
	
		GtkWidget	*createPluginsConnectionTabs();
		void		appendPluginsConnectionTab(IMPluginDaemon *proto);
			
		static gboolean cb_eventCallback(UserEventType type, gint command, gint lastResult, gpointer info, OptionsWindowItem_connections* self);
		static void		cb_setOtherNegativeChecked(GtkWidget *checkbox1, GtkWidget* checkbox2);
};


class OptionsWindowItem_plugins: public OptionsWindowItem
{
	public:
		OptionsWindowItem_plugins();
		~OptionsWindowItem_plugins();
	
		GtkWidget	*createTab();
	
	private:
		GtkListStore	*standardStore,
						*protoStore;	
	
		GtkWidget	*createPluginsList(GtkListStore **store);
	
		static void cb_loadedCheckboxToggled(GtkCellRenderer* t, gchar* pathString, OptionsWindowItem_plugins* self);
		static void cb_enabledCheckboxToggled(GtkCellRenderer* t, gchar* pathString, OptionsWindowItem_plugins* self);
		static gboolean cb_eventCallback(UserEventType type, gint command, gint lastResult, gpointer info, OptionsWindowItem_plugins* self);
};


class OptionsWindowItem_start: public OptionsWindowItem
{
	public:
		OptionsWindowItem_start();
		~OptionsWindowItem_start();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget	*autoLogonStatusList,
					*autoLogonEnabledCheckbox,
					*autoLogonInvisibleCheckbox;
};


class OptionsWindowItem_aresponse: public OptionsWindowItem
{
	public:
		OptionsWindowItem_aresponse();
		~OptionsWindowItem_aresponse();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget		*arTypeView,
						*arTemplatesView,
						*arTemplateEntryView,
						*arAutoAwayEnabledCheckbox,
						*arAutoNAEnabledCheckbox,
						*arAutoAwaySpinButton,
						*arAutoNASpinButton,
						*arAutoAwayTemplatesCombo,
						*arAutoNATemplatesCombo;
	
		GtkListStore	*arTypeStore,
						*lastStore;
		GtkTreeIter		lastIter;
		GtkTreeViewColumn *col;
		
		// callbacks
		static void cb_arTypeChanged(OptionsWindowItem_aresponse* self);
		static void cb_arTemplateChanged(OptionsWindowItem_aresponse* self);
		static void cb_arTemplatesViewEdited(GtkCellRendererText *cell, gchar *pathStr, gchar *newText, OptionsWindowItem_aresponse* self);
};


class OptionsWindowItem_themes: public OptionsWindowItem
{
	public:
		OptionsWindowItem_themes();
		~OptionsWindowItem_themes();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:	
		GtkWidget	*statusCombo,
					*eventCombo,
					*extendedCombo,
					*smileysCombo,
					
					*statusBox,
					*eventBox,
					*extendedBox,
					*smileysBox;
	
		GList		*statusIcons,
					*eventIcons,
					*extendedIcons;
	
		IconManagerSmiley	*smileys;
	
	
		void 		fillThemeCombos();
		GtkWidget	*createThemeCombo();
		void		showIcons(GtkWidget	*box, GList	*list);
		void		showSmileys();
		
		// callbacks
		static void cb_statusComboChanged(OptionsWindowItem_themes* self);
		static void cb_eventComboChanged(OptionsWindowItem_themes* self);
		static void cb_extendedComboChanged(OptionsWindowItem_themes* self);
		static void cb_smileysComboChanged(OptionsWindowItem_themes* self);
};


struct KeyCombination
{
	gchar	*name,
			*keyStr;
	guint 	keyVal;
	EggVirtualModifierType modifier;
};


struct KeyMetaInfo
{
	gchar	*descr,
			*confName;
};


class OptionsWindowItem_keys: public OptionsWindowItem
{
	public:
		OptionsWindowItem_keys();
		~OptionsWindowItem_keys();
	
		GtkWidget	*createTab();
		gboolean	applyChanges();
	
	private:
		GtkWidget			*keysView;
		GtkTreeStore		*keysStore;	
		GtkTreeViewColumn	*keysColumn;
		
		void fillCombinationsSection(gchar *section, KeyMetaInfo *cnt);
	
		static void cb_KeyCombinationEdited(GtkCellRenderer* renderer, 
											gchar* pathStr, 
											guint keyVal, 
											EggVirtualModifierType modifier, 
											guint keyCode,
											OptionsWindowItem_keys* self);
	
		static void cb_KeyCombinationCleared(OptionsWindowItem_keys* self,
											 gchar* pathStr);
	
		static void cb_keyEntryActivated(OptionsWindowItem_keys* self);
};


void o_cb_sensitivityToggleButtonClicked(GtkWidget* btn, GtkWidget* victim);

#endif /* _OPTIONSWINDOW_H */
