/***************************************************************************
 *            settings.h
 *
 *  Mon Jul  5 23:37:40 2004
 *  Copyright  2004  User
 *  Email
 ****************************************************************************/

#ifndef _SETTINGS_H
#define _SETTINGS_H

#include "licq_icqd.h"

// #include "licq_plugin.h"
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gobject/gvaluecollector.h>
		
	



#define ICQND_TYPE_COLOR G_TYPE_POINTER

#define GDK_COLOR_SET(col,r,g,b)\
	(col)->red = r;				\
	(col)->green = g;			\
	(col)->blue = b;

#define GDK_COLOR_SET_AV(rescol, col1, col2)			\
	(rescol)->red = ((col1)->red + (col2)->red)/2;		\
	(rescol)->green = ((col1)->green + (col2)->green)/2;\
	(rescol)->blue = ((col1)->blue + (col2)->blue)/2;

typedef void (*SettingChangedFunc)(gchar* sname, GList *values, gpointer data);


struct SettingsFunc
{
	SettingChangedFunc func;
	gpointer data;
};


struct SettingsProperty
{
	gchar  	*name;
	GValue 	*val;
	GList	*hatches;
};


struct SettingsSection
{
	gchar *name;
	GList *properties;  // a list of SettingsProperty
	GList *funcs;		// a list of SettingsFunc
};



class Settings
{
	public:
		Settings(gchar* filename);
		~Settings();	
	
		// installs new properties in [section]
		// parameterorder: gchar* name, GType, default value if there is no entry in the config
		void installProperties(gchar* sname, ...); 
	
		// changes a property for a specific section
		void setProperties(gboolean inform, gchar* sname, ...);
	
		// gets properties
		void getProperties(gchar* sname, ...);
		
		// installs a listener for changes of a specific section
		void installListener(gchar* sname, SettingChangedFunc func, gpointer data, gboolean prepend);	
	
		// removes a listener for a section
		void removeListenerByData(gpointer data);
	
		// installs a hatch. Whenever something is changed this variable gets changed too
		void installHatch(gchar* sname, gchar* propname, gpointer hatch);
	
		// removes a hatch. 
		void removeHatch(gchar* sname, gchar* propname, gpointer hatch);
		
		// calls all listeners for a specific section
		void callListeners(gchar* sname);
	
		// saves the Settings
		gboolean writeSettings();
	
	
		GtkTextTagTable	*textTags;				
	private:
	
		void	applyPropertyToHatch(GValue	*prop, gpointer hatch);
	
		void	loadTextTags();	
	
		CIniFile *licqFile;	
		GList	*sections;	// a list of settingSection
	
	
		static void cb_textTagsChangedCallback(gchar* sname, GList *values, Settings* self);
};



Settings* Settings_getSettings();
void	  Settings_setSettings(Settings *s);	


#endif /* _SETTINGS_H */
