/***************************************************************************
 *            icons.cpp
 *
 *  Tue Aug 16 23:52:18 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "icons.h"

IconManager* iconMan = NULL;

IconManager::IconManager()
{
	gchar *base,
		  *eventTheme,
		  *smileyTheme,
		  *statusTheme,
		  *extendedTheme,
		  *searchingPath;
	
	icons = NULL;
	eventIcons = NULL;
	extendedIcons = NULL;
	
	Settings_getSettings()->getProperties("appearance",
				  						  "EventTheme", &eventTheme,
				  						  "SmileyTheme", &smileyTheme,
				  						  "StatusTheme", &statusTheme,
				  						  "ExtendedTheme", &extendedTheme,
				  						  NULL);
	
	// load status icons
	base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, statusTheme);
	loadStatusIcons(base, NULL, TRUE);	
	g_free(base);
	
	// load event icons
	base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, eventTheme);
	loadEventIcons(base, NULL);
	g_free(base);
	
	// load extended icons
	base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, extendedTheme);
	loadExtendedIcons(base, NULL);
	g_free(base);
	
	// load smileys
	images.smileys = NULL;
	base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, smileyTheme);
	loadSmileys(base, &images.smileys, TRUE);
	g_free(base);

	// load the searching animation
	searchingPath = g_strdup_printf("%sicqnd/search.gif", SHARE_DIR);
	images.searchingAni = gtk_image_get_animation( GTK_IMAGE(gtk_image_new_from_file(searchingPath)));
	g_free(searchingPath);
	
	// append a listener for changed icons
	Settings_getSettings()->installListener("appearance", (SettingChangedFunc)this->cb_iconThemesChanged, this, TRUE);
}



IconManager::~IconManager()
{
	clearStatusIcons(&icons);
	clearSmileys(&images.smileys, TRUE);
	clearIcons(&eventIcons);
	clearIcons(&extendedIcons);
}



void IconManager::clearStatusIcons(GList** list)
{
	GList			*search,
					*searchi;
	IconManagerCat	*iconCat;
	IconManagerIcon	*icon;
	
	
	search = *list;
	while(search)
	{
		iconCat = (IconManagerCat*)search->data;
		
		searchi = iconCat->icons;
		while(searchi)
		{
			icon = (IconManagerIcon*)searchi->data;
			
			g_object_unref(G_OBJECT(icon->pb));
			g_object_unref(G_OBJECT(icon->apb));
			g_free(icon);
			
			searchi = searchi->next;
		}
		
		g_free(iconCat->protocolID);
		g_list_free(iconCat->icons);
		g_free(iconCat);
		
		search = search->next;
	}
	
	if (*list)
		g_list_free(*list);

	*list = NULL;	
}


void IconManager::clearIcons(GList** iconList)
{
	GList			*search;
	IconManagerIcon	*icon;
	
	if (!*iconList)
		return;
	
	for (search = *iconList; search; search = search->next)
	{
		icon = (IconManagerIcon*)search->data;
		g_object_unref(icon->pb);
		g_free(icon);
	}
	
	g_list_free(*iconList);
	*iconList = NULL;	
}


void IconManager::loadStatusIcons(gchar* base, GList** list, gboolean setStock)
{
	GDir			*dir,
					*iconDir;
	const gchar		*currentName;
	gchar			*iconPath,
					*iconName,
					*statusBase;
	gint			i;
	IconManagerCat	*cat;
	IconManagerIcon	*icon;
	GtkIconFactory	*iconFactory;
	GtkIconSet		*stockIcon; 
	GdkPixbuf		*alphaImg;
	
	struct _iconManLoad
	{
		gchar	*iconName;
		glong	status;
	} iconManStatus[] =
	{
		{"away.png", BUDDY_STATUS_AWAY},
		{"dnd.png", BUDDY_STATUS_DND},
		{"ffc.png", BUDDY_STATUS_FFC},
		{"invisible.png", BUDDY_STATUS_INV},
		{"invisible_set.png", BUDDY_INV_SET},
		{"na.png", BUDDY_STATUS_NA},
		{"occ.png", BUDDY_STATUS_OCC},
		{"offline.png", BUDDY_STATUS_OFFLINE},		
		{"online.png", BUDDY_STATUS_ONLINE}
	};
	
	if (!list)
		list = &icons;
	
	// eventually clear an old structure
	if (*list)
		clearStatusIcons(list);
	
	iconFactory = NULL;
	
	if (setStock)
		iconFactory = gtk_icon_factory_new();
	
	
	statusBase = g_strdup_printf("%s/status", base);
	dir = g_dir_open(statusBase, 0, NULL);

	while((currentName = g_dir_read_name(dir)))
	{
		iconPath = g_strdup_printf("%s/%s", statusBase, currentName);
		
		if (g_file_test(iconPath, G_FILE_TEST_IS_DIR))
		{
			cat = (IconManagerCat*)g_malloc0(sizeof(IconManagerCat));
			cat->protocolID = g_strdup(currentName);
			
			iconDir = g_dir_open(iconPath, 0, NULL);
			while((currentName = g_dir_read_name(iconDir)))
			{
				for (i=0; i<9; i++)
				{
					if (!strcmp(currentName, iconManStatus[i].iconName))
					{
						iconName = g_strdup_printf("%s/%s", iconPath, currentName);
						icon = (IconManagerIcon*)g_malloc0(sizeof(IconManagerIcon));
						icon->identifier = iconManStatus[i].status;
						icon->pb = gdk_pixbuf_new_from_file(iconName, NULL);
						cat->icons = g_list_append(cat->icons, icon);
						g_free(iconName);
						
						// add a alphaed pixbuf for invisible modes
						alphaImg = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
												  TRUE,
												  gdk_pixbuf_get_bits_per_sample(icon->pb),
												  gdk_pixbuf_get_width(icon->pb),
												  gdk_pixbuf_get_height(icon->pb));
						gdk_pixbuf_fill(alphaImg, 0x00000000);
						gdk_pixbuf_composite(icon->pb, alphaImg, 
											 0, 0, 
											 gdk_pixbuf_get_width(icon->pb),
											 gdk_pixbuf_get_height(icon->pb),
											 0.0f, 0.0f,
											 1.0f, 1.0f,
											 GDK_INTERP_NEAREST,
											 128);
						icon->apb = alphaImg;

						
						// also add an entry to the icon factory for the menu
						if (setStock)
						{
							iconName = g_strdup_printf("icqnd-status-%s-%ld", cat->protocolID, iconManStatus[i].status);
							stockIcon = gtk_icon_set_new_from_pixbuf(icon->pb);
							gtk_icon_factory_add(iconFactory, iconName, stockIcon); 			
							g_free(iconName);
						}
					}
				}				
			}
			
			g_dir_close(iconDir);
			
			*list = g_list_append(*list, cat);
			
		}
		
		g_free(iconPath);
	}
	
	g_free(statusBase);
	
	if (setStock)
		gtk_icon_factory_add_default(iconFactory);

	g_dir_close(dir);
}



GdkPixbuf* IconManager::getStatusPixbuf(gchar* protoID, gulong status, gboolean inv)
{
	GList	*search;
	IconManagerCat	*cat;
	IconManagerIcon	*icon;
	
	if (status == BUDDY_STATUS_INV)
	{
		status = BUDDY_STATUS_ONLINE;
		inv = TRUE;
	}
	
	search = icons;
	
	while(search)
	{
		cat = (IconManagerCat*)search->data;
		if (!g_ascii_strcasecmp(cat->protocolID, protoID))
			break;
		
		search = search->next;
	}
	
	if (!search)
		cat = (IconManagerCat*)icons->data;
	
	search = cat->icons;
	while(search)
	{
		icon = (IconManagerIcon*)search->data;
		if (icon->identifier == status)
		{
			if (inv)
				return icon->apb;
			else
				return icon->pb;
		}
		
		search = search->next;
	}
	
	return NULL;
}



GdkPixbuf* IconManager::getUserStatusPixbuf(IMUserDaemon* user)
{
	gchar	*proto;
	
	if (!user || !user->owner || !((IMOwnerDaemon*)user->owner)->protocol)
		return NULL;
	
	if (user->info->pluginID == LICQ_PPID)
	{
		if (isdigit(user->info->licqID[0]))
			proto = "icq";
		else
			proto = "aim";
	} else
		proto = ((IMOwnerDaemon*)user->owner)->protocol->licqName;
	
		return getStatusPixbuf(proto,
							   user->info->status,
							   user->info->isInvisible);
	
	return NULL;
}



GdkPixbuf* IconManager::getEventPixbuf(gint ev)
{
	GList			*search;
	IconManagerIcon	*ico;
	
	if (ev == -1)
		return NULL;
	
	search = eventIcons;
	
	while(search)
	{
		ico = (IconManagerIcon*)search->data;
		if (ico->identifier == ev)
			return ico->pb;
		
		search = search->next;
	}
	
	if (eventIcons)
		return ((IconManagerIcon*)eventIcons->data)->pb;
	else
		return NULL;	
}



GdkPixbuf* IconManager::getExtendedPixbuf(guint num)
{
	GList			*search;
	IconManagerIcon	*ico;
	
	for (search = extendedIcons; search; search = search->next)
	{
		ico = (IconManagerIcon*)search->data;
		if (ico->identifier == num)
			return ico->pb;
	}
	
	return NULL;
}



void IconManager::loadIcons(gchar *base, gchar *section, StockIconType* icons, GList **iconList)
{
	GtkIconFactory	*iconFactory; // the icon factory containing all icons
	GtkIconSet		*icon; 	      // an icon set (we use one pic for every size)
	GdkPixbuf		*pixbuf;
	gchar			*fileName;
	guint			i;
	IconManagerIcon	*newIcon;
	
	iconFactory = gtk_icon_factory_new();

				
	// load all icons into stock
	i=0;
	while(icons[i].fileName)
	{
		fileName = g_strdup_printf("%s/%s/%s", base, section, icons[i].fileName);
		
		// Check whether the file exists. If not -> use the default dir
		if (!g_file_test(fileName, G_FILE_TEST_EXISTS))
		{
			g_free(fileName);
			fileName = g_strdup_printf("%sicqnd/default/%s/%s", SHARE_DIR, section, icons[i].fileName);
		}
		
		if (g_file_test(fileName, G_FILE_TEST_EXISTS))
		{
			pixbuf = gdk_pixbuf_new_from_file(fileName, NULL);
			icon = gtk_icon_set_new_from_pixbuf(pixbuf);
			gtk_icon_factory_add(iconFactory, icons[i].stockID, icon); 
			
			if (iconList)
			{
				newIcon = (IconManagerIcon*)g_malloc0(sizeof(IconManagerIcon));
				newIcon->pb = pixbuf;
				newIcon->identifier = icons[i].id;
				
				*iconList = g_list_append(*iconList, newIcon);
			}
			
		}
		else
			fprintf(stderr, "image file not found: %s\n", fileName);
		
		g_free(fileName);
		
		i++;
	}
	
	gtk_icon_factory_add_default(iconFactory);
}




void IconManager::loadEventIcons(gchar *base, GList **list)
{	
	StockIconType icons[] = {	
		{ICON_MESSAGE, "message.png", USER_EVENT_MESSAGE},		
		{ICON_CHAT, "chat.png", USER_EVENT_CHAT},
		{ICON_FILE, "file.png", USER_EVENT_FILE},
		{ICON_URL, "url.png", USER_EVENT_URL},		
		{ICON_HISTORY, "history.png", 0},
		{ICON_INFO, "info.png", 0},
		{ICON_SMILEYS, "smiley.png", 0},
		{ICON_SYSMSG, "sysmsg.png", 0},
		{ICON_SHOWSMILEYS, "showsmileys.png", 0},
		{ICON_SHOWURLS, "showurls.png", 0},
		{ICON_ARROW_CLOSED1, "arrow_c1.png", ARROW_CLOSED1},
		{ICON_ARROW_CLOSED2, "arrow_c2.png", ARROW_CLOSED2},
		{ICON_ARROW_OPEN1, "arrow_o1.png", ARROW_OPEN1},
		{ICON_ARROW_OPEN2, "arrow_o2.png", ARROW_OPEN2},
		{ICON_KEYRING, "keyring.png", 0},
		{ICON_AUTH, "auth.png", 0},
		{ICON_CONTACTS, "contacts.png", 0},
		{ICON_SHOWOFFLINE, "showoffline.png", 0},
		{ICON_GROUPS, "showgroups.png", 0},
		{ICON_OPENGROUP, "arrow_c1.png", 0},
		{ICON_CLOCK, "clock.png", 0},
		{NULL, NULL}		
		};
		
		if (!list)
			list = &eventIcons;

		clearIcons(list);	
		loadIcons(base, "events", icons, list);
}


void IconManager::loadExtendedIcons(gchar *base, GList** list)
{
	StockIconType icons[] = {	
		{ICON_BIRTHDAY, "aresponse.png", IET_ARESPONSE},
		{ICON_TYPING1, "birthday.png", IET_BIRTHDAY},
		{ICON_TYPING2, "cell.png", IET_CELL},
		{ICON_TYPING3, "phone.png", IET_PHONE},	
		{ICON_TYPING3, "typing.png", IET_TYPING},			
		{NULL, NULL},		
		};	
		
		if (!list)
			list = &extendedIcons;
		
		clearIcons(list);
		
		
		loadIcons(base, "extended", icons, list);
	
}



void IconManager::loadSmileys(gchar *base, IconManagerSmiley** smileysList, gboolean buildTree)
{
	FILE		*tokensFile;
	gchar		*tokensName;
	gchar		*buf,
				*smileyName;
	gulong		size,
				pos,
				count,
				smileyPos;
	guint		i;
	GString		*smileyToken,
				*imageName,
				*smileyCount;
	gboolean 	hasToken;
	GList		*search,
				*genuineIcons;

	if (!smileysList)
		smileysList = &images.smileys;
	
	if (*smileysList)
		clearSmileys(smileysList, buildTree);
	
	// open the rules file	
	tokensName = g_strdup_printf("%s/smileys/tokens.conf", base);
	tokensFile = fopen(tokensName, "r");
	if (!tokensFile)
	{
		fprintf(stderr, "Settings::loadSmileys(): Could not open smiley tokens file %s!\n", tokensName);
		g_free(tokensName);
		smileysList = NULL;
		smileyTree = NULL;
		return;
	}
	
	fseek(tokensFile, 0, SEEK_END);
	size = ftell(tokensFile);
	rewind(tokensFile);

	buf = (gchar*)g_malloc0(size);
	fread(buf, 1, size, tokensFile);
	
	fclose(tokensFile);
	g_free(tokensName);

	// read the file content -> create the smileys tree and load the images	
	pos = 0;
	smileyCount = g_string_new("");
	hasToken = FALSE;
	
	// get the smiley count
	while(buf[pos] != '\n')
		smileyCount = g_string_append_c(smileyCount, buf[pos++]);

	count = strtoul(smileyCount->str, NULL, 10);
	g_string_free(smileyCount, TRUE);
	
	// is the count valid
	if (!count)
	{
		fprintf(stderr, "Settings::loadSmileys(): Corrupted tokens.conf file. The beginning of the file must be the smiley count.\n");
		g_free(buf);
		fclose(tokensFile);
		smileysList = NULL;
		smileyTree = NULL;
		return;
	}
		
	smileyToken = g_string_new("");
	imageName = g_string_new("");
	*smileysList = (IconManagerSmiley*)g_malloc0(sizeof(IconManagerSmiley)*(count+1));
	smileyPos = 0;
	genuineIcons = NULL;
	pos++;
	
	while(pos<size && smileyPos<count)
	{
		// smiley tokens and image names are separated by tabs
		if (buf[pos] == '\t')
			hasToken = TRUE;
		
		// a line is over -> we should have a new pair
		if (buf[pos] == '\n')
		{
			// is the line valid? if not -> return
			if (!strlen(smileyToken->str) || !strlen(imageName->str))
			{
				fprintf(stderr, "Settings::loadSmileys(): Corrupted tokens.conf file. Format must be\n		smileytoken[TAB]image name[RETURN]!");
				g_free(tokensName);
				g_free(buf);
				g_string_free(smileyToken, TRUE);
				g_string_free(imageName, TRUE);
				fclose(tokensFile);
				smileysList = NULL;
				smileyTree = NULL;
				return;
			}
			
			smileyName = g_strdup_printf("%s/smileys/%s", base, imageName->str);
			
			(*smileysList)[smileyPos].token = g_strdup(smileyToken->str);
			(*smileysList)[smileyPos].icon = gdk_pixbuf_new_from_file(smileyName, NULL);
			
			search = genuineIcons;
			while(search)
			{
				if (!strcmp(smileyName, (gchar*)search->data))
					break;
				search = search->next;
			}
			
			if (!search)
				genuineIcons = g_list_append(genuineIcons, g_strdup(smileyName));
			
			(*smileysList)[smileyPos].isFirstIcon = (search == NULL);
			
			g_free(smileyName);
			
			smileyToken = g_string_assign(smileyToken, "");
			imageName = g_string_assign(imageName, "");
			smileyPos++;
			hasToken = FALSE;
			pos++;
		}
		
		if (!hasToken)
			smileyToken = g_string_append_c(smileyToken, buf[pos]);
		else
			if (buf[pos] != '\t' && buf[pos] != '\n')
				imageName = g_string_append_c(imageName, buf[pos]);
		
		pos++;
	}
	
	g_free(buf);
	g_string_free(smileyToken, TRUE);
	g_string_free(imageName, TRUE);
	
	g_list_foreach(genuineIcons, (GFunc)g_free, NULL);
	g_list_free(genuineIcons);
	
	
		
	// create a tree containing the smiley information
	// to avoid looking for every character of a smiley token on every text
	// character a tree is generated.
	// if we have eg. the smileys ":) :( :P", the first node only contains
	// one element: ":" and three branches: ")", "(", "P". This way programs
	// have only to look for the ":" and only THEN look for other possibilities
	// for this smiley.

	
	if (buildTree)
	{
		smileyTree = (SmileysNode*)g_malloc0(sizeof(SmileysNode));
	
		smileyTree->summary = g_string_new("");
		for (i=0;i<count;i++)
			smileyTree->similies = g_list_append(smileyTree->similies, GUINT_TO_POINTER(i));
	
		// create the smileys tree
		createSmileyTree(smileyTree, 0);
	}
}


void IconManager::clearSmileys(IconManagerSmiley** smileysList, gboolean clearTree)
{
	guint		i;
	
	if (!*smileysList)
		return;
	
	// clear the images
	i = 0;
	while((*smileysList)[i].token)
	{
		g_free((*smileysList)[i].token);
		g_object_unref(G_OBJECT((*smileysList)[i].icon));
		
		i++;
	}
	
	g_free(*smileysList);
	
	*smileysList = NULL;
	
	if (clearTree)
	{
		// clear the smileys tree
		g_list_free(clearSmileysTree(smileyTree));
		smileyTree = NULL;
	}
}



GList* IconManager::clearSmileysTree(SmileysNode *node, GList* deletedChildren)
{
	GList	*children;

	// check whether this child was already deleted
	for (children = deletedChildren; children; children = children->next)
		if (children->data == node)
			return deletedChildren;
		
	if (node && node->children)
	{
		deletedChildren = g_list_append(deletedChildren, node);
		children = node->children;
		while(children)
		{
			deletedChildren = clearSmileysTree((SmileysNode*)children->data, deletedChildren);
			children = children->next;
		}
		
		g_list_free(node->similies);
		g_list_free(node->children);
		g_string_free(node->summary, TRUE);
		g_free(node);
	}
	
	return deletedChildren;
}



void IconManager::createSmileyTree(SmileysNode *node, guchar pos)
{
	SmileysNode *node2;
	GList		*search,
				*search2;
	guint		index;
	
	
	search = node->similies;
	
	while(search)
	{
		index = GPOINTER_TO_UINT(search->data);
		
		if (images.smileys[index].token && strlen(images.smileys[index].token)>pos)
		{					
			search2 = node->children;
	
			while(search2)
			{
				node2 = (SmileysNode*)search2->data;
			
				if (node2->character == images.smileys[index].token[pos])
					break;
				
				search2 = search2->next;
			}
			
			if (!search2)
			{
				node2 = (SmileysNode*)g_malloc0(sizeof(SmileysNode));
				node2->summary = g_string_new("");
				node2->character = images.smileys[index].token[pos];
				if (pos+1 == (guchar)strlen(images.smileys[index].token))
					node2->smileysIndex = index;
				else
					node2->smileysIndex = -1;
				
				g_string_append_c(node->summary, images.smileys[index].token[pos]);
				node->children = g_list_append(node->children, node2);
				
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index));
				
			} else
				node2->similies = g_list_append(node2->similies, GUINT_TO_POINTER(index)); 
		}
		
		search = search->next;
	}
	
	
	search = node->children;
	while(search)
	{
		node2 = (SmileysNode*)search->data;
		
		createSmileyTree(node2, pos+1);
		search = search->next;
	}
	

	// add cross references so the user doesn't need to type a "nose" when typing a smiley
	search = node->children;
	while(search)
	{
		node2 = (SmileysNode*)search->data;

		if (node2->character == '-')
			node->children = g_list_concat(node->children, g_list_copy(node2->children)); 			
		
		// createSmileyTree(node2, pos+1);
		search = search->next;
	} 
}



void IconManager::cb_iconThemesChanged(gchar* sname, GList *values, IconManager* self)
{
	SettingsProperty	*prop;
	gchar				*base;

	while(values)
	{
		prop = (SettingsProperty*)values->data;
		
		if (!strcmp(prop->name, "EventTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, g_value_get_string(prop->val));
			self->loadEventIcons(base, NULL);
			g_free(base);
		} else
		if (!strcmp(prop->name, "SmileyTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, g_value_get_string(prop->val));
			self->loadSmileys(base, &self->images.smileys, TRUE);
			g_free(base);	
		} else
		if (!strcmp(prop->name, "StatusTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, g_value_get_string(prop->val));
			self->loadStatusIcons(base, NULL, TRUE);
			g_free(base);
		} else
		if (!strcmp(prop->name, "ExtendedTheme"))
		{
			base = g_strdup_printf("%sicqnd/%s", SHARE_DIR, g_value_get_string(prop->val));
			self->loadExtendedIcons(base, NULL);
			g_free(base);
		}
		
		values = values->next;
	}
}



IconManager* i_getIcons()
{
	if (!iconMan)
		iconMan = new IconManager();
	
	return iconMan;
}
